from typing import Optional

from cpl_core.application import StartupExtensionABC
from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironmentABC
from cpl_discord import get_discord_collection
from cpl_discord.discord_event_types_enum import DiscordEventTypesEnum

from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from modules.admin.command.restart_command import RestartCommand
from modules.admin.command.shutdown_command import ShutdownCommand
from modules.base.command.afk_command import AFKCommand
from modules.base.command.help_command import HelpCommand
from modules.base.command.info_command import InfoCommand
from modules.base.command.ping_command import PingCommand
from modules.base.events.base_on_command_error_event import BaseOnCommandErrorEvent
from modules.base.events.base_on_command_event import BaseOnCommandEvent
from modules.moderator.command.purge_command import PurgeCommand
from modules.base.command.user_info_command import UserInfoCommand
from modules.base.events.base_on_member_join_event import BaseOnMemberJoinEvent
from modules.base.events.base_on_member_remove_event import BaseOnMemberRemoveEvent
from modules.base.events.base_on_message_event import BaseOnMessageEvent
from modules.base.events.base_on_voice_state_update_event import BaseOnVoiceStateUpdateEvent
from modules.boot_log.boot_log_on_ready_event import BootLogOnReadyEvent
from modules.database.database_on_ready_event import DatabaseOnReadyEvent
from modules.permission.events.permission_on_member_update_event import PermissionOnMemberUpdateEvent
from modules.permission.events.permission_on_ready_event import PermissionOnReadyEvent


class StartupDiscordExtension(StartupExtensionABC):

    def __init__(self):
        self._feature_flags: Optional[FeatureFlagsSettings] = None

    def configure_configuration(self, config: ConfigurationABC, env: ApplicationEnvironmentABC):
        self._feature_flags = config.get_configuration(FeatureFlagsSettings)

    def configure_services(self, services: ServiceCollectionABC, env: ApplicationEnvironmentABC):
        services.add_discord()
        dc = get_discord_collection(services)
        """modules"""
        if self._feature_flags.admin_module:
            """ commands """
            dc.add_command(RestartCommand)
            dc.add_command(ShutdownCommand)
            """ events """

        if self._feature_flags.base_module:
            """ commands """
            dc.add_command(AFKCommand)
            dc.add_command(HelpCommand)
            dc.add_command(InfoCommand)
            dc.add_command(PingCommand)
            dc.add_command(UserInfoCommand)
            """ events """
            dc.add_event(DiscordEventTypesEnum.on_command.value, BaseOnCommandEvent)
            dc.add_event(DiscordEventTypesEnum.on_command_error.value, BaseOnCommandErrorEvent)
            dc.add_event(DiscordEventTypesEnum.on_member_join.value, BaseOnMemberJoinEvent)
            dc.add_event(DiscordEventTypesEnum.on_member_join.value, BaseOnMemberRemoveEvent)
            dc.add_event(DiscordEventTypesEnum.on_message.value, BaseOnMessageEvent)
            dc.add_event(DiscordEventTypesEnum.on_voice_state_update.value, BaseOnVoiceStateUpdateEvent)

        if self._feature_flags.database_module:
            """ commands """
            """ events """
            dc.add_event(DiscordEventTypesEnum.on_ready.value, DatabaseOnReadyEvent)

        if self._feature_flags.moderator_module:
            """ commands """
            dc.add_command(PurgeCommand)
            """ events """

        if self._feature_flags.permission_module:
            """ commands """
            """ events """
            dc.add_event(DiscordEventTypesEnum.on_ready.value, PermissionOnReadyEvent)
            dc.add_event(DiscordEventTypesEnum.on_member_update.value, PermissionOnMemberUpdateEvent)

        # has to be last!
        if self._feature_flags.boot_log_module:
            """ commands """
            """ events """
            dc.add_event(DiscordEventTypesEnum.on_ready.value, BootLogOnReadyEvent)
