from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.migration_abc import MigrationABC
from bot_data.db_context import DBContext


class InitialMigration(MigrationABC):

    def __init__(self, logger: DatabaseLogger, db: DBContext):
        self._logger = logger
        self._db = db
        self._cursor = db.cursor

    def upgrade(self):
        self._logger.debug(__name__, 'Running upgrade')

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `MigrationHistory` (
                `MigrationId` VARCHAR(255),
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                PRIMARY KEY(`MigrationId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `Servers` (
                `ServerId` BIGINT NOT NULL AUTO_INCREMENT,
                `DiscordServerId` BIGINT NOT NULL,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                PRIMARY KEY(`ServerId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `Users` (
                `UserId` BIGINT NOT NULL AUTO_INCREMENT,
                `DiscordId` BIGINT NOT NULL,
                `XP` BIGINT NOT NULL DEFAULT 0,
                `ServerId` BIGINT,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`ServerId`) REFERENCES Servers(`ServerId`),
                PRIMARY KEY(`UserId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `Clients` (
                `ClientId` BIGINT NOT NULL AUTO_INCREMENT,
                `DiscordClientId` BIGINT NOT NULL,
                `SentMessageCount` BIGINT NOT NULL DEFAULT 0,
                `ReceivedMessageCount` BIGINT NOT NULL DEFAULT 0,
                `DeletedMessageCount` BIGINT NOT NULL DEFAULT 0,
                `ReceivedCommandsCount` BIGINT NOT NULL DEFAULT 0,
                `MovedUsersCount` BIGINT NOT NULL DEFAULT 0,
                `ServerId` BIGINT,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`ServerId`) REFERENCES Servers(`ServerId`),
                PRIMARY KEY(`ClientId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `KnownUsers` (
                `KnownUserId` BIGINT NOT NULL AUTO_INCREMENT,
                `DiscordId` BIGINT NOT NULL,
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                PRIMARY KEY(`KnownUserId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `UserJoinedServers` (
                `JoinId` BIGINT NOT NULL AUTO_INCREMENT,
                `UserId` BIGINT NOT NULL,
                `JoinedOn` DATETIME(6) NOT NULL,
                `LeavedOn` DATETIME(6),
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`UserId`) REFERENCES Users(`UserId`),
                PRIMARY KEY(`JoinId`)
            );
            """)
        )

        self._cursor.execute(
            str(f"""
            CREATE TABLE IF NOT EXISTS `UserJoinedVoiceChannel` (
                `JoinId` BIGINT NOT NULL AUTO_INCREMENT,
                `UserId` BIGINT NOT NULL,
                `DiscordChannelId` BIGINT NOT NULL,
                `JoinedOn` DATETIME(6) NOT NULL,
                `LeavedOn` DATETIME(6),
                `CreatedAt` DATETIME(6),
                `LastModifiedAt` DATETIME(6),
                FOREIGN KEY (`UserId`) REFERENCES Users(`UserId`),
                PRIMARY KEY(`JoinId`)
            );
            """)
        )

    def downgrade(self):
        self._cursor.execute('DROP TABLE `Servers`;')
        self._cursor.execute('DROP TABLE `Users`;')
        self._cursor.execute('DROP TABLE `Clients`;')
        self._cursor.execute('DROP TABLE `KnownUsers`;')
        self._cursor.execute('DROP TABLE `UserJoinedServers`;')
        self._cursor.execute('DROP TABLE `UserJoinedVoiceChannel`;')
