from cpl_core.configuration import ConfigurationABC
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord import VoiceChannel
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.logging.command_logger import CommandLogger
from modules.base.configuration.base_server_settings import BaseServerSettings


class AFKCommand(DiscordCommandABC):

    def __init__(
            self,
            logger: CommandLogger,
            config: ConfigurationABC,
            message_service: MessageServiceABC,
            bot: DiscordBotServiceABC,
            client_utils: ClientUtilsServiceABC,
            translate: TranslatePipe
    ):
        DiscordCommandABC.__init__(self)

        self._logger = logger
        self._config = config
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils
        self._t = translate

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.command()
    @commands.guild_only()
    async def afk(self, ctx: Context):
        self._logger.debug(__name__, f'Received command afk {ctx}')
        self._client_utils.received_command(ctx.guild.id)
        settings: BaseServerSettings = self._config.get_configuration(f'BaseServerSettings_{ctx.guild.id}')

        if ctx.author.voice is None or ctx.author.voice.channel is None:
            await self._message_service.send_ctx_msg(ctx, self._t.transform('modules.base.afk_command_channel_missing_message'))
            self._logger.trace(__name__, f'Finished afk command')
            return

        self._bot.loop.create_task(self._message_service.send_ctx_msg(ctx, self._t.transform('modules.base.afk_command_move_message')))
        channel: VoiceChannel = ctx.guild.get_channel(settings.afk_command_channel_id)
        try:
            await ctx.author.move_to(channel)
            self._client_utils.moved_user(ctx.guild.id)
        except Exception as e:
            self._logger.error(__name__, f'Cannot move user {ctx.author.id} to channel {ctx.channel.id}', e)
            await self._message_service.send_ctx_msg(ctx, self._t.transform('common.no_permission_message'))

        self._logger.trace(__name__, f'Finished afk command')
